"""
Exports AMBIGUOUS_EASTASIAN table keyed by supporting unicode version level.

This code generated by wcwidth/bin/update-tables.py on 2026-01-18 23:27:15 UTC.
"""
# pylint: disable=duplicate-code
AMBIGUOUS_EASTASIAN = {
    '17.0.0': (
        # Source: EastAsianWidth-17.0.0.txt
        # Date: 2025-07-24, 00:12:54 GMT
        #
        (0x000a1, 0x000a1,),  # Inverted Exclamation Mark
        (0x000a4, 0x000a4,),  # Currency Sign
        (0x000a7, 0x000a8,),  # Section Sign            ..Diaeresis
        (0x000aa, 0x000aa,),  # Feminine Ordinal Indicator
        (0x000ad, 0x000ae,),  # Soft Hyphen             ..Registered Sign
        (0x000b0, 0x000b4,),  # Degree Sign             ..Acute Accent
        (0x000b6, 0x000ba,),  # Pilcrow Sign            ..Masculine Ordinal Indica
        (0x000bc, 0x000bf,),  # Vulgar Fraction One Quar..Inverted Question Mark
        (0x000c6, 0x000c6,),  # Latin Capital Letter Ae
        (0x000d0, 0x000d0,),  # Latin Capital Letter Eth
        (0x000d7, 0x000d8,),  # Multiplication Sign     ..Latin Capital Letter O W
        (0x000de, 0x000e1,),  # Latin Capital Letter Tho..Latin Small Letter A Wit
        (0x000e6, 0x000e6,),  # Latin Small Letter Ae
        (0x000e8, 0x000ea,),  # Latin Small Letter E Wit..Latin Small Letter E Wit
        (0x000ec, 0x000ed,),  # Latin Small Letter I Wit..Latin Small Letter I Wit
        (0x000f0, 0x000f0,),  # Latin Small Letter Eth
        (0x000f2, 0x000f3,),  # Latin Small Letter O Wit..Latin Small Letter O Wit
        (0x000f7, 0x000fa,),  # Division Sign           ..Latin Small Letter U Wit
        (0x000fc, 0x000fc,),  # Latin Small Letter U With Diaeresis
        (0x000fe, 0x000fe,),  # Latin Small Letter Thorn
        (0x00101, 0x00101,),  # Latin Small Letter A With Macron
        (0x00111, 0x00111,),  # Latin Small Letter D With Stroke
        (0x00113, 0x00113,),  # Latin Small Letter E With Macron
        (0x0011b, 0x0011b,),  # Latin Small Letter E With Caron
        (0x00126, 0x00127,),  # Latin Capital Letter H W..Latin Small Letter H Wit
        (0x0012b, 0x0012b,),  # Latin Small Letter I With Macron
        (0x00131, 0x00133,),  # Latin Small Letter Dotle..Latin Small Ligature Ij
        (0x00138, 0x00138,),  # Latin Small Letter Kra
        (0x0013f, 0x00142,),  # Latin Capital Letter L W..Latin Small Letter L Wit
        (0x00144, 0x00144,),  # Latin Small Letter N With Acute
        (0x00148, 0x0014b,),  # Latin Small Letter N Wit..Latin Small Letter Eng
        (0x0014d, 0x0014d,),  # Latin Small Letter O With Macron
        (0x00152, 0x00153,),  # Latin Capital Ligature O..Latin Small Ligature Oe
        (0x00166, 0x00167,),  # Latin Capital Letter T W..Latin Small Letter T Wit
        (0x0016b, 0x0016b,),  # Latin Small Letter U With Macron
        (0x001ce, 0x001ce,),  # Latin Small Letter A With Caron
        (0x001d0, 0x001d0,),  # Latin Small Letter I With Caron
        (0x001d2, 0x001d2,),  # Latin Small Letter O With Caron
        (0x001d4, 0x001d4,),  # Latin Small Letter U With Caron
        (0x001d6, 0x001d6,),  # Latin Small Letter U With Diaeresis And Macron
        (0x001d8, 0x001d8,),  # Latin Small Letter U With Diaeresis And Acute
        (0x001da, 0x001da,),  # Latin Small Letter U With Diaeresis And Caron
        (0x001dc, 0x001dc,),  # Latin Small Letter U With Diaeresis And Grave
        (0x00251, 0x00251,),  # Latin Small Letter Alpha
        (0x00261, 0x00261,),  # Latin Small Letter Script G
        (0x002c4, 0x002c4,),  # Modifier Letter Up Arrowhead
        (0x002c7, 0x002c7,),  # Caron
        (0x002c9, 0x002cb,),  # Modifier Letter Macron  ..Modifier Letter Grave Ac
        (0x002cd, 0x002cd,),  # Modifier Letter Low Macron
        (0x002d0, 0x002d0,),  # Modifier Letter Triangular Colon
        (0x002d8, 0x002db,),  # Breve                   ..Ogonek
        (0x002dd, 0x002dd,),  # Double Acute Accent
        (0x002df, 0x002df,),  # Modifier Letter Cross Accent
        (0x00391, 0x003a1,),  # Greek Capital Letter Alp..Greek Capital Letter Rho
        (0x003a3, 0x003a9,),  # Greek Capital Letter Sig..Greek Capital Letter Ome
        (0x003b1, 0x003c1,),  # Greek Small Letter Alpha..Greek Small Letter Rho
        (0x003c3, 0x003c9,),  # Greek Small Letter Sigma..Greek Small Letter Omega
        (0x00401, 0x00401,),  # Cyrillic Capital Letter Io
        (0x00410, 0x0044f,),  # Cyrillic Capital Letter ..Cyrillic Small Letter Ya
        (0x00451, 0x00451,),  # Cyrillic Small Letter Io
        (0x02010, 0x02010,),  # Hyphen
        (0x02013, 0x02016,),  # En Dash                 ..Double Vertical Line
        (0x02018, 0x02019,),  # Left Single Quotation Ma..Right Single Quotation M
        (0x0201c, 0x0201d,),  # Left Double Quotation Ma..Right Double Quotation M
        (0x02020, 0x02022,),  # Dagger                  ..Bullet
        (0x02024, 0x02027,),  # One Dot Leader          ..Hyphenation Point
        (0x02030, 0x02030,),  # Per Mille Sign
        (0x02032, 0x02033,),  # Prime                   ..Double Prime
        (0x02035, 0x02035,),  # Reversed Prime
        (0x0203b, 0x0203b,),  # Reference Mark
        (0x0203e, 0x0203e,),  # Overline
        (0x02074, 0x02074,),  # Superscript Four
        (0x0207f, 0x0207f,),  # Superscript Latin Small Letter N
        (0x02081, 0x02084,),  # Subscript One           ..Subscript Four
        (0x020ac, 0x020ac,),  # Euro Sign
        (0x02103, 0x02103,),  # Degree Celsius
        (0x02105, 0x02105,),  # Care Of
        (0x02109, 0x02109,),  # Degree Fahrenheit
        (0x02113, 0x02113,),  # Script Small L
        (0x02116, 0x02116,),  # Numero Sign
        (0x02121, 0x02122,),  # Telephone Sign          ..Trade Mark Sign
        (0x02126, 0x02126,),  # Ohm Sign
        (0x0212b, 0x0212b,),  # Angstrom Sign
        (0x02153, 0x02154,),  # Vulgar Fraction One Thir..Vulgar Fraction Two Thir
        (0x0215b, 0x0215e,),  # Vulgar Fraction One Eigh..Vulgar Fraction Seven Ei
        (0x02160, 0x0216b,),  # Roman Numeral One       ..Roman Numeral Twelve
        (0x02170, 0x02179,),  # Small Roman Numeral One ..Small Roman Numeral Ten
        (0x02189, 0x02189,),  # Vulgar Fraction Zero Thirds
        (0x02190, 0x02199,),  # Leftwards Arrow         ..South West Arrow
        (0x021b8, 0x021b9,),  # North West Arrow To Long..Leftwards Arrow To Bar O
        (0x021d2, 0x021d2,),  # Rightwards Double Arrow
        (0x021d4, 0x021d4,),  # Left Right Double Arrow
        (0x021e7, 0x021e7,),  # Upwards White Arrow
        (0x02200, 0x02200,),  # For All
        (0x02202, 0x02203,),  # Partial Differential    ..There Exists
        (0x02207, 0x02208,),  # Nabla                   ..Element Of
        (0x0220b, 0x0220b,),  # Contains As Member
        (0x0220f, 0x0220f,),  # N-ary Product
        (0x02211, 0x02211,),  # N-ary Summation
        (0x02215, 0x02215,),  # Division Slash
        (0x0221a, 0x0221a,),  # Square Root
        (0x0221d, 0x02220,),  # Proportional To         ..Angle
        (0x02223, 0x02223,),  # Divides
        (0x02225, 0x02225,),  # Parallel To
        (0x02227, 0x0222c,),  # Logical And             ..Double Integral
        (0x0222e, 0x0222e,),  # Contour Integral
        (0x02234, 0x02237,),  # Therefore               ..Proportion
        (0x0223c, 0x0223d,),  # Tilde Operator          ..Reversed Tilde
        (0x02248, 0x02248,),  # Almost Equal To
        (0x0224c, 0x0224c,),  # All Equal To
        (0x02252, 0x02252,),  # Approximately Equal To Or The Image Of
        (0x02260, 0x02261,),  # Not Equal To            ..Identical To
        (0x02264, 0x02267,),  # Less-than Or Equal To   ..Greater-than Over Equal
        (0x0226a, 0x0226b,),  # Much Less-than          ..Much Greater-than
        (0x0226e, 0x0226f,),  # Not Less-than           ..Not Greater-than
        (0x02282, 0x02283,),  # Subset Of               ..Superset Of
        (0x02286, 0x02287,),  # Subset Of Or Equal To   ..Superset Of Or Equal To
        (0x02295, 0x02295,),  # Circled Plus
        (0x02299, 0x02299,),  # Circled Dot Operator
        (0x022a5, 0x022a5,),  # Up Tack
        (0x022bf, 0x022bf,),  # Right Triangle
        (0x02312, 0x02312,),  # Arc
        (0x02460, 0x024e9,),  # Circled Digit One       ..Circled Latin Small Lett
        (0x024eb, 0x0254b,),  # Negative Circled Number ..Box Drawings Heavy Verti
        (0x02550, 0x02573,),  # Box Drawings Double Hori..Box Drawings Light Diago
        (0x02580, 0x0258f,),  # Upper Half Block        ..Left One Eighth Block
        (0x02592, 0x02595,),  # Medium Shade            ..Right One Eighth Block
        (0x025a0, 0x025a1,),  # Black Square            ..White Square
        (0x025a3, 0x025a9,),  # White Square Containing ..Square With Diagonal Cro
        (0x025b2, 0x025b3,),  # Black Up-pointing Triang..White Up-pointing Triang
        (0x025b6, 0x025b7,),  # Black Right-pointing Tri..White Right-pointing Tri
        (0x025bc, 0x025bd,),  # Black Down-pointing Tria..White Down-pointing Tria
        (0x025c0, 0x025c1,),  # Black Left-pointing Tria..White Left-pointing Tria
        (0x025c6, 0x025c8,),  # Black Diamond           ..White Diamond Containing
        (0x025cb, 0x025cb,),  # White Circle
        (0x025ce, 0x025d1,),  # Bullseye                ..Circle With Right Half B
        (0x025e2, 0x025e5,),  # Black Lower Right Triang..Black Upper Right Triang
        (0x025ef, 0x025ef,),  # Large Circle
        (0x02605, 0x02606,),  # Black Star              ..White Star
        (0x02609, 0x02609,),  # Sun
        (0x0260e, 0x0260f,),  # Black Telephone         ..White Telephone
        (0x0261c, 0x0261c,),  # White Left Pointing Index
        (0x0261e, 0x0261e,),  # White Right Pointing Index
        (0x02640, 0x02640,),  # Female Sign
        (0x02642, 0x02642,),  # Male Sign
        (0x02660, 0x02661,),  # Black Spade Suit        ..White Heart Suit
        (0x02663, 0x02665,),  # Black Club Suit         ..Black Heart Suit
        (0x02667, 0x0266a,),  # White Club Suit         ..Eighth Note
        (0x0266c, 0x0266d,),  # Beamed Sixteenth Notes  ..Music Flat Sign
        (0x0266f, 0x0266f,),  # Music Sharp Sign
        (0x0269e, 0x0269f,),  # Three Lines Converging R..Three Lines Converging L
        (0x026bf, 0x026bf,),  # Squared Key
        (0x026c6, 0x026cd,),  # Rain                    ..Disabled Car
        (0x026cf, 0x026d3,),  # Pick                    ..Chains
        (0x026d5, 0x026e1,),  # Alternate One-way Left W..Restricted Left Entry-2
        (0x026e3, 0x026e3,),  # Heavy Circle With Stroke And Two Dots Above
        (0x026e8, 0x026e9,),  # Black Cross On Shield   ..Shinto Shrine
        (0x026eb, 0x026f1,),  # Castle                  ..Umbrella On Ground
        (0x026f4, 0x026f4,),  # Ferry
        (0x026f6, 0x026f9,),  # Square Four Corners     ..Person With Ball
        (0x026fb, 0x026fc,),  # Japanese Bank Symbol    ..Headstone Graveyard Symb
        (0x026fe, 0x026ff,),  # Cup On Black Square     ..White Flag With Horizont
        (0x0273d, 0x0273d,),  # Heavy Teardrop-spoked Asterisk
        (0x02776, 0x0277f,),  # Dingbat Negative Circled..Dingbat Negative Circled
        (0x02b56, 0x02b59,),  # Heavy Oval With Oval Ins..Heavy Circled Saltire
        (0x03248, 0x0324f,),  # Circled Number Ten On Bl..Circled Number Eighty On
        (0x0e000, 0x0f8ff,),  # (nil)
        (0x0fffd, 0x0fffd,),  # Replacement Character
        (0x1f100, 0x1f10a,),  # Digit Zero Full Stop    ..Digit Nine Comma
        (0x1f110, 0x1f12d,),  # Parenthesized Latin Capi..Circled Cd
        (0x1f130, 0x1f169,),  # Squared Latin Capital Le..Negative Circled Latin C
        (0x1f170, 0x1f18d,),  # Negative Squared Latin C..Negative Squared Sa
        (0x1f18f, 0x1f190,),  # Negative Squared Wc     ..Square Dj
        (0x1f19b, 0x1f1ac,),  # Squared Three D         ..Squared Vod
        (0xf0000, 0xffffd,),  # (nil)
        (0x100000, 0x10fffd,),  # (nil)
    ),
}
